@@ -49,7 +49,7 @@ class tdcoa():
     secretpath = ''
     filesetpath = ''
     outputpath = ''
-    version = "0.3.3.2"
+    version = "0.3.6.2"
 
     # log settings
     logs =  []
@@ -72,7 +72,7 @@ class tdcoa():
     settings = {}
 
 
-    def __init__(self, approot='.', printlog=True, config='config.yaml', secrets='secrets.yaml', filesets='filesets.yaml', pyfile_only=False):
+    def __init__(self, approot='.', printlog=True, config='config.yaml', secrets='secrets.yaml', filesets='filesets.yaml', make_setup_file=False):
         self.bufferlog = True
         self.printlog = printlog
         self.approot = os.path.join('.', approot)
@@ -88,19 +88,19 @@ class tdcoa():
         self.skipgit = False
         self.skipdbs = False
 
-        if pyfile_only:
+        if make_setup_file:
             self.pyfile_setup(filename='setup.py')
-        else:
-            # stub in config.yaml, secrets.yaml, if missing
-            if not os.path.isfile(self.configpath):
-                self.log('missing config.yaml', 'creating %s' %self.configpath)
-                self.yaml_config(writefile=True, skipgit=self.skipgit, skipdbs=self.skipdbs)
-            if not os.path.isfile(self.secretpath):
-                self.log('missing secrets.yaml', 'creating %s' %self.secretpath)
-                self.yaml_secrets(writefile=True)
-            # filesets.yaml is validated at download time
 
-            self.reload_config()
+        # stub in config.yaml, secrets.yaml, if missing
+        if not os.path.isfile(self.configpath):
+            self.log('missing config.yaml', 'creating %s' %self.configpath)
+            self.yaml_config(writefile=True, skipgit=self.skipgit, skipdbs=self.skipdbs)
+        if not os.path.isfile(self.secretpath):
+            self.log('missing secrets.yaml', 'creating %s' %self.secretpath)
+            self.yaml_secrets(writefile=True)
+        # filesets.yaml is validated at download time
+
+        self.reload_config()
 
 
 
@@ -246,6 +246,7 @@ class tdcoa():
         cy.append('')
         cy.append('')
         cy.append('folders:')
+        cy.append('  override:  "0_override"')
         cy.append('  download:  "1_download"')
         cy.append('  sql:       "2_sql_store"')
         cy.append('  run:       "3_ready_to_run"')
@@ -811,8 +812,8 @@ class tdcoa():
         # check and set required Folders
         self.log('loading dictionary', 'folders')
         self.folders = configyaml['folders']
-        self.check_setting(self.folders, required_item_list=['download','sql','run','output'],
-                           defaults=['1_download','2_sql_store','3_ready_to_run','4_output'])
+        self.check_setting(self.folders, required_item_list=['download','sql','run','output','override'],
+                           defaults=['1_download','2_sql_store','3_ready_to_run','4_output','0_override'])
 
         # check and set required Settings
         self.log('loading dictionary', 'settings')
@@ -1000,10 +1001,11 @@ class tdcoa():
                                     self.log('  saving file to', savefile)
                                     with open(savefile, 'w') as fh:
                                         fh.write(filecontent)
-
         self.log('\ndone!')
         self.log('time', str(dt.datetime.now()))
 
+        self.copy_download_to_sql()
+
 
 
 
@@ -1015,6 +1017,8 @@ class tdcoa():
         downloadpath = os.path.join(self.approot, self.folders['download'])
         sqlpath = os.path.join(self.approot, self.folders['sql'])
 
+        self.recursively_delete_subfolders(sqlpath)
+
         for sysname, sysobject in self.systems.items():
             self.log('processing system', sysname)
             if self.dict_active(sysobject, also_contains_key='filesets'):
@@ -1038,17 +1042,87 @@ class tdcoa():
 
 
 
+    def apply_override(self, override_folder='', target_folder=''):
+        self.log('applying file override')
 
-    def prepare_sql(self, sqlfolder=''):
+        # apply folder default locations
+        if override_folder=='': override_folder = self.folders['override']
+        override_folder = os.path.join(self.approot, override_folder)
+
+        if target_folder=='': target_folder = self.folders['sql']
+        target_folder = os.path.join(self.approot, target_folder)
+
+        self.log(' override folder', override_folder)
+        self.log(' target_folder', target_folder)
+
+        copyops={}
+        allfiles = []
+
+        # map files found in override folder
+        logdone=False
+        reloadconfig=False
+        for fo, subfos, files in os.walk(override_folder):
+            if fo == override_folder:
+                self.log('\nprocessing files found in override root')
+                self.log('these files replace any matching filename, regardless of subfolder location')
+                for file in files:
+                    if file=='config.yaml' or file=='secrets.yaml':
+                        copyops[os.path.join(self.approot,file)] = os.path.join(override_folder, file)
+                        reloadconfig=True
+                        self.log('  config file found, reload imminent',file)
+                    elif file[:1] != '.':
+                        allfiles.append(file)
+                        self.log('  root file found',file)
+            else:
+                if os.path.basename(fo)[:1] != '.':
+                    if not logdone:
+                        logdone=True
+                        self.log('\nprocessing files found in override subfolders')
+                        self.log('these files only replace filenames found in matching subfolders (and overrides root files)')
+
+                    for file in files:
+                        if file[:1] != '.':
+                            specfile = os.path.join(fo, file).replace(override_folder,'.')
+                            keydestfile = os.path.join(target_folder, specfile)
+                            keydestfo = os.path.dirname(keydestfile)
+                            if os.path.exists(keydestfo):
+                                copyops[keydestfile] = os.path.join(override_folder, specfile)
+                                self.log('  subfolder file found', specfile)
+                            else:
+                                self.log('target folder does not exist',keydestfo,warning=True)
+
+        # search for matching allfiles by crawling the target_folder
+        for fo, subfos, files in os.walk(target_folder):
+            for file in files:
+                keydestfile = os.path.join(fo, file)
+                if file in allfiles:
+                    copyops[keydestfile] = os.path.join(override_folder, file)
+
+        # perform final copy:
+        self.log('\nperform override file copy:')
+        for dstpath, srcpath in copyops.items():
+            self.log(' source:  %s' %srcpath)
+            self.log(' target:  %s' %dstpath)
+            shutil.copyfile(srcpath, dstpath)
+
+        if reloadconfig:  self.reload_config()
+        self.log('\napply override complete!')
+
+
+
+
+    def prepare_sql(self, sqlfolder='', override_folder=''):
         self.log('prepare_sql started', header=True)
         self.log('time',str(dt.datetime.now()))
 
         if sqlfolder != '':
-            self.log('sql folder override', sqlfolder)
+            self.log('sql folder', sqlfolder)
             self.folders['sql'] = sqlfolder
         self.log(' sql folder', self.folders['sql'])
         self.log(' run folder', self.folders['run'])
 
+        self.apply_override(target_folder = sqlfolder, override_folder=override_folder)
+
         # clear pre-existing subfolders in "run" directory (file sets)
         self.log('empty run folder entirely')
         self.recursively_delete_subfolders(os.path.join(self.approot, self.folders['run']))
